"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBabelUtils = void 0;
const path_1 = require("path");
const utils_1 = require("@modern-js/utils");
// compatible with windows path
const formatPath = (originPath) => {
    if ((0, path_1.isAbsolute)(originPath)) {
        return originPath.split(path_1.sep).join('/');
    }
    return originPath;
};
const getPluginItemName = (item) => {
    if (typeof item === 'string') {
        return formatPath(item);
    }
    if (Array.isArray(item) && typeof item[0] === 'string') {
        return formatPath(item[0]);
    }
    return null;
};
const addPlugins = (plugins, config) => {
    if (config.plugins) {
        config.plugins.push(...plugins);
    }
    else {
        config.plugins = plugins;
    }
};
const addPresets = (presets, config) => {
    if (config.presets) {
        config.presets.push(...presets);
    }
    else {
        config.presets = presets;
    }
};
const removePlugins = (plugins, config) => {
    if (!config.plugins) {
        return;
    }
    const removeList = (0, utils_1.ensureArray)(plugins);
    config.plugins = config.plugins.filter((item) => {
        const name = getPluginItemName(item);
        if (name) {
            return !removeList.find(removeItem => name.includes(removeItem));
        }
        return true;
    });
};
const removePresets = (presets, config) => {
    if (!config.presets) {
        return;
    }
    const removeList = (0, utils_1.ensureArray)(presets);
    config.presets = config.presets.filter((item) => {
        const name = getPluginItemName(item);
        if (name) {
            return !removeList.find(removeItem => name.includes(removeItem));
        }
        return true;
    });
};
const modifyPresetOptions = (presetName, options, presets = []) => {
    presets.forEach((preset, index) => {
        // 1. ['@babel/preset-env', ...]
        if (Array.isArray(preset)) {
            if (typeof preset[0] === 'string' &&
                (0, utils_1.normalizeToPosixPath)(preset[0]).includes(presetName)) {
                preset[1] = {
                    ...(preset[1] || {}),
                    ...options,
                    // `options` is specific to different presets
                };
            }
        }
        else if (typeof preset === 'string' &&
            (0, utils_1.normalizeToPosixPath)(preset).includes(presetName)) {
            // 2. '@babel/preset-env'
            presets[index] = [preset, options];
        }
    });
};
const getBabelUtils = (config) => {
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    const noop = () => { };
    return {
        addPlugins: (plugins) => addPlugins(plugins, config),
        addPresets: (presets) => addPresets(presets, config),
        removePlugins: (plugins) => removePlugins(plugins, config),
        removePresets: (presets) => removePresets(presets, config),
        // `addIncludes` and `addExcludes` are noop functions by default,
        // It can be overridden by `extraBabelUtils`.
        addIncludes: noop,
        addExcludes: noop,
        // Compat `presetEnvOptions` and `presetReactOptions` in Eden.
        modifyPresetEnvOptions: (options) => modifyPresetOptions('@babel/preset-env', options, config.presets || []),
        modifyPresetReactOptions: (options) => modifyPresetOptions('@babel/preset-react', options, config.presets || []),
    };
};
exports.getBabelUtils = getBabelUtils;
